<?php

namespace OCA\GrueneGroupfolderManagement\AppInfo;

use OC\Memcache\APCu;
use OCA\DAV\Events\CalendarShareUpdatedEvent;
use OCA\GrueneGroupfolderManagement\ApiClient;
use OCA\GrueneGroupfolderManagement\Cache\DefaultTtlDecorator;
use OCA\GrueneGroupfolderManagement\Cache\RandomizeTtlDecorator;
use OCA\GrueneGroupfolderManagement\Groups\GroupBackend;
use OCA\GrueneGroupfolderManagement\Groups\SherpaGroupBackend;
use OCA\GrueneGroupfolderManagement\Listener\ShareListener;
use OCA\GrueneGroupfolderManagement\Middleware\ErrorMiddleware;
use OCA\GrueneGroupfolderManagement\Security\AuthorizationService;
use OCA\GrueneGroupfolderManagement\Security\DivisionVoter;
use OCA\GrueneGroupfolderManagement\Settings\Admin;
use OCP\AppFramework\App;
use OCP\AppFramework\Bootstrap\IBootContext;
use OCP\AppFramework\Bootstrap\IBootstrap;
use OCP\AppFramework\Bootstrap\IRegistrationContext;
use OCP\EventDispatcher\IEventDispatcher;
use OCP\ICache;
use OCP\ICacheFactory;
use OCP\IConfig;
use OCP\IGroupManager;
use OCP\IUserSession;
use OCP\Share\Events\ShareCreatedEvent;
use OCP\Util;
use Psr\Container\ContainerInterface;
use Respect\Validation\Factory;

class Application extends App implements IBootstrap {
	public const APP_ID = 'gruenegroupfoldermanagement';

	private ?ICache $cache = null;

	public function __construct(array $urlParams = array()) {
		parent::__construct(self::APP_ID, $urlParams);

		$this->cache = $this->createCache();
	}

	private function createCache(): ICache {
		$prefix = self::APP_ID . '_';

		$cacheFactory = $this->getContainer()->get(ICacheFactory::class);
		if ($cacheFactory->isLocalCacheAvailable()) {
			$cache = $cacheFactory->createLocal($prefix);
			return new DefaultTtlDecorator(new RandomizeTtlDecorator($cache, 1.1));
		}

		if (function_exists('apcu_fetch')) {
			$cache = new APCu(self::APP_ID . '_');
			return new DefaultTtlDecorator(new RandomizeTtlDecorator($cache, 1.1));
		}

		return null;
	}


	public function register(IRegistrationContext $context): void {
		require_once __DIR__ . '/../../vendor/autoload.php';

		/** @var IEventDispatcher */
		$dispatcher = $this->getContainer()->get(IEventDispatcher::class);
		$dispatcher->addServiceListener(ShareCreatedEvent::class, ShareListener::class);
		$dispatcher->addServiceListener(CalendarShareUpdatedEvent::class, ShareListener::class);
		$dispatcher->addListener(
			'OCA\Files::loadAdditionalScripts',
			function () {
				Util::addScript(self::APP_ID, 'gruenegroupfoldermanagement-main');
			}
		);

		$context->registerService(ApiClient::class, function (ContainerInterface $c) {
			$config = $c->get(IConfig::class);
			$client = new ApiClient(
				$config->getAppValue(
					self::APP_ID,
					'api_server',
					Admin::DEFAULT_API_SERVER
				),
				$config->getAppValue(
					self::APP_ID,
					'api_user',
				),
				$config->getAppValue(
					self::APP_ID,
					'api_password',
				),
			);
			$client->setCache($this->cache);
			return $client;
		});


		$context->registerService(AuthorizationService::class, function (ContainerInterface $c) {
			$service = new AuthorizationService($c->get(IUserSession::class));
			$service->registerVoter($c->get(DivisionVoter::class));
			return $service;
		});

		$this->setupValidation();
		$context->registerMiddleware(ErrorMiddleware::class);
	}

	private function setupValidation() {
		Factory::setDefaultInstance(
			(new Factory())
				->withRuleNamespace('OCA\\GrueneGroupfolderManagement\\Validation\\Rules')
				->withExceptionNamespace('OCA\\GrueneGroupfolderManagement\\Validation\\Exceptions')
				->withTranslator(function ($message) {
					$translations = [
						"{{name}} must be valid email" => "{{name}} muss eine gültige Email-Adresse sein."
					];
					return $translations[$message] ?? $message;
				})
		);
	}


	public function boot(IBootContext $context): void {
		$groupManager = $this->getContainer()->get(IGroupManager::class);
		$groupManager->addBackend($this->getContainer()->get(GroupBackend::class));
		$groupManager->addBackend($this->getContainer()->get(SherpaGroupBackend::class));
	}
}
