<?php

namespace OCA\GrueneGroupfolderManagement\Db;

use OCA\GrueneGroupfolderManagement\Manager\DivisionManager;
use OCP\DB\QueryBuilder\IQueryBuilder;
use OCP\IDBConnection;
use OCP\AppFramework\Db\DoesNotExistException;
use OCP\AppFramework\Db\MultipleObjectsReturnedException;
use OCP\AppFramework\Db\QBMapper;

class GroupMapper extends QBMapper {
	public const TABLE_GROUPS = 'gruenegfm_division_groups';
	public const VIEW_GROUP_FOLDERS = 'gruenegfm_group_folders';

	/**
	 * @var DivisionManager
	 */
	protected $divisionManager;

	public function __construct(IDBConnection $db, DivisionManager $divisionManager) {
		parent::__construct($db, self::TABLE_GROUPS);
		$this->divisionManager = $divisionManager;
	}

	/**
	 * @param int|null $limit
	 * @param int|null $limit
	 *
	 * @return Group[]
	 */
	public function findAll(int $limit = null, int $offset = null) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*')
		   ->from(self::TABLE_GROUPS);
		if ($limit !== null) {
			$qb->setMaxResults($limit);
		}

		if ($offset !== null) {
			$qb->setFirstResult($offset);
		}
		return $this->setDivisions($this->findEntities($qb));
	}



	public function findMany(
		int $limit = 20,
		int $offset = 0,
		?array $divisions = null,

	) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*')
		   ->from(self::TABLE_GROUPS)
		   ->setMaxResults($limit)
		   ->setFirstResult($offset)
		;

		if ($divisions) {
			$qb->where(
				$qb->expr()->in('division_key', $qb->createNamedParameter($divisions, IQueryBuilder::PARAM_STR_ARRAY))
			);
		}

		return $this->findEntities($qb);
	}

	/**
	 * @param string $id
	 *
	 * @return null|Group
	 */
	public function find(int $id) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUPS);
		$qb->where(
			$qb->expr()->eq('id', $qb->createNamedParameter($id, IQueryBuilder::PARAM_INT))
		);
		try {
			return $this->setDivisions(array($this->findEntity($qb)))[0];
		} catch(DoesNotExistException $e) {
			return null;
		}
	}

	/**
	 * @param string $id
	 *
	 * @return Group[]
	 */
	public function findByDivision(string $id) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUPS);
		$qb->where(
			$qb->expr()->eq('division_key', $qb->createNamedParameter($id))
		);
		return $this->setDivisions($this->findEntities($qb));
	}

	/**
	 * @param int $division
	 * @param string $group
	 *
	 * @return Group
	 */
	public function findByDivisionAndName(int $division, string $group) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUPS);
		$qb->where(
			$qb->expr()->eq('division_key', $qb->createNamedParameter($division))
		);
		$qb->andWhere(
			$qb->expr()->eq($qb->func()->lower('name'), $qb->createNamedParameter(mb_strtolower($group)))
		);
		try {
			return $this->setDivisions(array($this->findEntity($qb)))[0];
		} catch (DoesNotExistException $e) {
			return null;
		}
	}


	/**
	 * Get group beneath this divisions group folder
	 *
	 * @param string $groupFolder the group folders mount point
	 * @param string $divisionName name of division
	 * @return Group division group
	 * @throws MultipleObjectsReturnedException — if more than one item exist
	 * @throws DoesNotExistException — if the item does not exist
	 */
	public function getSingleDivisionByGroupFolder(string $groupFolder, string $divisionName) {
		$query = $this->db->getQueryBuilder();
		$query->select(
			'id',
			'division_key',
			'division_acl_permission',
			'division_acl_mask',
			'member_acl_permission',
			'member_acl_mask',
			'admin_acl_permission',
			'admin_acl_mask',
			'type',
			'name',
			'active'
		)
			->from(self::VIEW_GROUP_FOLDERS, 'g')
			->innerJoin(
				'g',
				self::TABLE_GROUPS,
				'd',
				$query->expr()->eq('organization_key', 'division_key')
			)
			->where($query->expr()->eq('mount_point', $query->createNamedParameter($groupFolder)))
			->andWhere($query->expr()->eq('name', $query->createNamedParameter($divisionName)));
		return $this->findEntity($query);
	}


	/**
	 * First fetch group id by its directory id
	 * Then try to fetch the group instance
	 *
	 * @param string $divisionKey alias gliederungs id
	 * @param string $foreignKey directory id
	 * @return Group
	 * @throws DoesNotExistException — if the item does not exist
	 * @throws MultipleObjectsReturnedException — if more than one item exist
	 */
	public function getGroupByFolderId(string $divisionKey, string $foreignKey) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*')
			->from('gruenegfm_mapping')
			->where($qb->expr()->eq('division_key', $qb->createNamedParameter($divisionKey)))
			->andWhere($qb->expr()->eq('foreign_key', $qb->createNamedParameter($foreignKey)));
		$result = $qb->executeQuery();
		$row = $result->fetch();
		if (!$row) {
			throw new DoesNotExistException('Group not found');
		}
		$groupId = (int)str_replace('group_', '', $row['index']);
		$query = $this->db->getQueryBuilder();
		$query->select('*')
			->from(self::TABLE_GROUPS)
			->where($query->expr()->eq('id', $query->createNamedParameter($groupId)));
		return $this->findEntity($query);
	}


	/**
	 * @param Group[] $groups
	 *
	 * @return Group[]
	 */
	private function setDivisions(array $groups) {
		foreach ($groups as $group) {
			$division = $this->divisionManager->find($group->getDivisionKey());
			if ($division) {
				$group->setDivision($division);
			}
		}
		return $groups;
	}
}
