<?php

namespace OCA\GrueneGroupfolderManagement\Db;

use OCP\DB\QueryBuilder\IQueryBuilder;
use OCP\IDBConnection;
use OCP\AppFramework\Db\DoesNotExistException;
use OCP\AppFramework\Db\QBMapper;

class GroupMemberMapper extends QBMapper {
	public const TYPE_MEMBER = 'member';

	public const TYPE_ADMIN = 'admin';

	public const TABLE_GROUP_MEMBERS = 'gruenegfm_groups_members';

	/**
	 * @var GroupMapper
	 */
	protected $groupMapper;

	public function __construct(IDBConnection $db, GroupMapper $groupMapper) {
		parent::__construct($db, self::TABLE_GROUP_MEMBERS);
		$this->groupMapper = $groupMapper;
	}

	/**
	 * @param string $id
	 * @param string $type
	 *
	 * @return GroupMember[]|null
	 */
	public function findMembersByGroupAndType(string $id, string $type) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUP_MEMBERS);
		$qb->where(
			$qb->expr()->eq('group_id', $qb->createNamedParameter($id))
		);
		$qb->andWhere(
			$qb->expr()->eq('type', $qb->createNamedParameter($type))
		);
		return $this->setGroups($this->findEntities($qb));
	}

	/**
	 * @param string $groupId
	 *
	 * @return GroupMember[]|null
	 */
	public function findMembersByGroup(string $groupId) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUP_MEMBERS);
		$qb->where(
			$qb->expr()->eq('group_id', $qb->createNamedParameter($groupId))
		);
		return $this->setGroups($this->findEntities($qb));
	}

	/**
	 * @return GroupMember[]|null
	 */
	public function findMembersInGroupByType(Group $group, string $type) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUP_MEMBERS);
		$qb->where(
			$qb->expr()->eq('group_id', $qb->createNamedParameter($group->getId()))
		);
		$qb->andWhere(
			$qb->expr()->eq('mapping_type', $qb->createNamedParameter('group'))
		);
		$qb->andWhere(
			$qb->expr()->eq('type', $qb->createNamedParameter($type))
		);
		return $this->setGroups($this->findEntities($qb));
	}

	/**
	 * @return GroupMember[]|null
	 */
	public function searchUserInGroupByType(
		string $search,
		Group $group,
		string $type,
		?int $limit = null,
		?int $offset = null
	) {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUP_MEMBERS);
		$qb->where(
			$qb->expr()->eq('group_id', $qb->createNamedParameter($group->getId()))
		);
		$qb->andWhere(
			$qb->expr()->eq('mapping_type', $qb->createNamedParameter('user'))
		);
		$qb->andWhere(
			$qb->expr()->eq('type', $qb->createNamedParameter($type))
		);
		$qb->andWhere(
			$qb->expr()->like('mapping_id', $qb->createNamedParameter('%'.$search.'%'))
		);
		if ($limit !== null) {
			$qb->setMaxResults($limit);
		}
		if ($offset !== null) {
			$qb->setFirstResult($offset);
		}
		return $this->setGroups($this->findEntities($qb));
	}

	/**
	 * @return GroupMember[]|null
	 */
	public function findGroupsByMappingIdAndType(array $uids, string $mappingType, string $type = null) {
		if (empty($uids)) {
			return [];
		}
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUP_MEMBERS);
		$qb->where(
			$qb->expr()->eq('mapping_type', $qb->createNamedParameter($mappingType))
		);
		if ($type !== null) {
			$qb->andWhere(
				$qb->expr()->eq('type', $qb->createNamedParameter($type))
			);
		}
		$exprs = [];
		foreach ($uids as $uid) {
			$exprs[] = $qb->expr()->eq('mapping_id', $qb->createNamedParameter($uid));
		}
		$qb->andWhere(
			call_user_func_array([$qb->expr(), 'orX'], $exprs)
		);
		return $this->setGroups($this->findEntities($qb));
	}


	/**
	 * Checks if entity exists and returns it, creates new instance otherwise
	 *
	 * @param Group $group
	 * @param string $mappingType
	 * @param string $mappingId
	 * @param string $type
	 * @return GroupMember
	 */
	public function getGroupMember(Group $group, string $mappingType, string $mappingId, string $type) {
		try {
			if ($group->getId() === null) {
				throw new DoesNotExistException('Group has no id assigned by database, so it cant exist');
			}
			$query = $this->db->getQueryBuilder();
			$query->select('*');
			$query->from(self::TABLE_GROUP_MEMBERS)
				->where($query->expr()->eq('group_id', $query->createNamedParameter($group->getId())))
				->andWhere($query->expr()->eq('mapping_type', $query->createNamedParameter($mappingType)))
				->andWhere($query->expr()->eq('mapping_id', $query->createNamedParameter($mappingId)))
			->andWhere($query->expr()->eq('type', $query->createNamedParameter($type)));
			return $this->findEntity($query);
		} catch(DoesNotExistException $e) {
			$groupMember = new GroupMember();
			$groupMember->setGroupId($group->getId());
			$groupMember->setMappingType($mappingType);
			$groupMember->setMappingId($mappingId);
			$groupMember->setType($type);
			$groupMember->setGroup($group);
			return $groupMember;
		}
	}


	/**
	 * Deletes all group member rows that belong to the passed group id
	 *
	 * @param int $groupId
	 * @return void
	 */
	public function deleteMembersByGroupId(int $groupId) {
		// TODO replace database column group id by foreign key with cascade on delete
		$query = $this->db->getQueryBuilder();
		$query->delete(self::TABLE_GROUP_MEMBERS)
			->where($query->expr()->eq('group_id', $query->createNamedParameter($groupId)));
		$query->executeStatement();
	}


	/**
	 * Delete all rows but not the supplied ids
	 *
	 * @param array $ids
	 * @param int $groupId
	 * @return void
	 */
	public function deleteMembersByIds(array $ids, int $groupId, bool $canEditAdminGroup) {
		$query = $this->db->getQueryBuilder();
		$query->delete(self::TABLE_GROUP_MEMBERS)
			->where($query->expr()->eq('group_id', $query->createNamedParameter($groupId)));
		if (count($ids) > 0) {
			$query->andWhere($query->expr()->notIn('id', $query->createNamedParameter($ids, IQueryBuilder::PARAM_INT_ARRAY)));
		}
		if (!$canEditAdminGroup) {
			// dont delete admin group members
			$query->andWhere($query->expr()->neq('type', $query->createNamedParameter('admin')));
		}
		$query->execute();
	}

	/**
	 *
	 * @return null|Group
	 */
	public function get(int $id): ?GroupMember {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*');
		$qb->from(self::TABLE_GROUP_MEMBERS);
		$qb->where(
			$qb->expr()->eq('id', $qb->createNamedParameter($id, IQueryBuilder::PARAM_INT))
		);

		try {
			return $this->findEntity($qb);
		} catch (DoesNotExistException) {
			return null;
		}
	}

	/**
	 * @return GroupMember[]
	 */
	public function findMany(
		int $limit = 20,
		int $offset = 0,
		?int $groupId = null,
		?string $mappingType = null,
		?string $mappingId = null,
		?string $type = null,
	): array {
		$qb = $this->db->getQueryBuilder();
		$qb->select('*')
			->from(self::TABLE_GROUP_MEMBERS)
			->setMaxResults($limit)
			->setFirstResult($offset)
		;

		if ($groupId) {
			$qb->andWhere(
				$qb->expr()->eq('group_id', $qb->createNamedParameter($groupId))
			);
		}
		if ($mappingType) {
			$qb->andWhere(
				$qb->expr()->eq('mapping_type', $qb->createNamedParameter($mappingType))
			);
		}
		if ($mappingId) {
			$qb->andWhere(
				$qb->expr()->eq('mapping_id', $qb->createNamedParameter($mappingId))
			);
		}
		if ($type) {
			$qb->andWhere(
				$qb->expr()->eq('type', $qb->createNamedParameter($type))
			);
		}

		return $this->findEntities($qb);
	}

	public function findOne(
		int $groupId,
		?string $mappingType = null,
		?string $mappingId = null,
		?string $type = null,
	): ?GroupMember {
		$members = $this->findMany(
			groupId: $groupId,
			limit: 1,
			mappingType: $mappingType,
			mappingId: $mappingId,
			type: $type
		);

		return $members[0] ?? null;
	}


	/**
	 * @var GroupMember[]
	 *
	 * @return GroupMember[]
	 */
	private function setGroups(array $members) {
		foreach ($members as $member) {
			$member->setGroup($this->groupMapper->find($member->getGroupId()));
		}
		return $members;
	}
}
