<?php

namespace OCA\GrueneGroupfolderManagement\Manager;

use OCA\GrueneGroupfolderManagement\ApiClient;
use OCA\GrueneGroupfolderManagement\Groups\GroupName;
use OCA\GrueneGroupfolderManagement\Model\DivisionGroup;
use OCA\GrueneGroupfolderManagement\Model\Division;
use OCA\GrueneGroupfolderManagement\Model\SAML\DivisionRole;
use OCP\IGroupManager;
use OCP\IUser;

class SherpaManager {
	/**
	 * @var ApiClient
	 */
	protected $apiClient;

	/**
	 * @var DivisionManager
	 */
	protected $divisionManager;

	/**
	 * @var IGroupManager
	 */
	protected $groupManager;

	public function __construct(
		ApiClient $apiClient,
		DivisionManager $divisionManager,
		IGroupManager $groupManager
	) {
		$this->apiClient = $apiClient;
		$this->divisionManager = $divisionManager;
		$this->groupManager = $groupManager;
	}

	/**
	 * @return Division[]
	 */
	public function getDivisions(IUser $user) : iterable {
		$groups = $this->groupManager->getUserGroupIds($user);
		foreach ($groups as $group) {
			$divisionRoles = GroupName::parseSAMLRole($group);
			foreach ($divisionRoles as $divisionRole) {
				$division = $this->divisionManager->find($divisionRole->getDivisionId());
				if ($division !== null) {
					yield $division;
				}
			}
		}

		yield from [];
	}

	/**
	 * @param string[] $groups
	 * @return Division[]
	 */
	public function getDivisionsByGroupIds(array $groups) : iterable {
		$map = [];

		foreach ($groups as $group) {
			$divisionRoles = GroupName::parseSAMLRole($group);

			foreach ($divisionRoles as $divisionRole) {
				$division = $this->divisionManager->find($divisionRole->getDivisionId());
				if ($division !== null) {
					$key = $division->getOrganizationalUuid();
					if (in_array($key, $map)) {
						continue;
					}
					$map[] = $key;

					yield $division;
				}
			}
		}

		yield from [];
	}

	/**
	 * @param string[] $types
	 * @return DivisionGroup[]
	 */
	public function getDivisionRolesByDivisionAndTypes(Division $division, array $types) : iterable {
		$map = [];

		foreach ($types as $type) {
			foreach ($this->apiClient->groupsByTypeAndLevel(
				$division->getOrganization(),
				$type,
				$division->getLevel()
			) as $sherpaGroup) {
				if (!$sherpaGroup->isActive()) {
					continue;
				}

				$key = $division->getOrganizationalUuid() . $sherpaGroup->getSlug();
				if (in_array($key, $map)) {
					continue;
				}
				$map[] = $key;

				yield new DivisionGroup($division, $sherpaGroup);
			}
		}

		yield from [];
	}

	/**
	 * @param string[] $groups
	 * @return DivisionGroup[]
	 */
	public function getDivisionRolesByGroupIds(array $groups) : iterable {
		$map = [];

		foreach ($groups as $group) {
			$divisionRoles = GroupName::parseSAMLRole($group);

			foreach ($divisionRoles as $divisionRole) {
				if (!($divisionRole instanceof DivisionRole)) {
					continue;
				}

				$division = $this->divisionManager->find($divisionRole->getDivisionId());
				if ($division !== null) {
					foreach ($this->apiClient->groupsBySherpaIdAndLevel(
						$division->getOrganization(),
						$divisionRole->getRoleId(),
						$division->getLevel()
					) as $sherpaGroup) {
						if (!$sherpaGroup->isActive()) {
							continue;
						}

						$key = $division->getOrganizationalUuid() . $sherpaGroup->getSlug();
						if (in_array($key, $map)) {
							continue;
						}
						$map[] = $key;

						yield new DivisionGroup($division, $sherpaGroup);
					}
				}
			}
		}

		yield from [];
	}

	/**
	 * @return DivisionGroup[]
	 */
	public function getDivisionRoles(IUser $user) : iterable {
		yield from $this->getDivisionRolesByGroupIds(
			$this->groupManager->getUserGroupIds($user)
		);
	}

	/**
	 * @return DivisionGroup[]
	 */
	public function filterDivisionRoles(IUser $user, string $type) : iterable {
		foreach ($this->getDivisionRoles($user) as $divisionGroup) {
			if ($divisionGroup->getGroup()->getType() == $type) {
				yield $divisionGroup;
			}
		}

		yield from [];
	}

	/**
	 * @return \Generator
	 */
	public function findByDivision(Division $division) : iterable {
		// TODO add cache
		foreach ($this->apiClient->groupsByLevel($division->getOrganization(), $division->getLevel()) as $group) {
			yield new DivisionGroup($division, $group);
		}

		yield from [];
	}

	public function findByDivisionAndSlug(Division $division, string $sherpaSlug) : ?DivisionGroup {
		// TODO add cache
		$group = $this->apiClient->groupBySlugAndLevel($division->getOrganization(), $sherpaSlug, $division->getLevel());

		if ($group === null) {
			return null;
		}

		return new DivisionGroup($division, $group);
	}
}
